<?php
/**
 * @brief		donateTopDonors Widget
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	donate
 * @since		31 Jul 2018
 */

namespace IPS\donate\widgets;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * donateTopDonors Widget
 */
class _donateTopDonors extends \IPS\Widget\PermissionCache
{
	/**
	 * @brief	Widget Key
	 */
	public $key = 'donateTopDonors';
	
	/**
	 * @brief	App
	 */
	public $app = 'donate';
		
	/**
	 * @brief	Plugin
	 */
	public $plugin = '';

    /**
     * Specify widget configuration
     *
     * @param	null|\IPS\Helpers\Form	$form	Form object
     * @return	\IPS\Helpers\Form
     */
    public function configuration( &$form=null )
    {
        if ( $form === null )
        {
            $form = new \IPS\Helpers\Form;
        }

        $form->add( new \IPS\Helpers\Form\Number( 'donateTopDonors_limit', isset( $this->configuration['donateTopDonors_limit'] ) ? $this->configuration['donateTopDonors_limit'] : 5, TRUE, array( 'max' => 30 ) ) );
        $form->add( new \IPS\Helpers\Form\Select( 'donateTopDonors_timeframe', isset( $this->configuration['donateTopDonors_timeframe'] ) ? $this->configuration['donateTopDonors_timeframe'] : 'month', TRUE, array( 'options' => array( 'week' => 'week', 'month' => 'month', 'year' => 'year', 'all' => 'alltime' ) ) ) );

        return $form;
    }

    /**
     * Render a widget
     *
     * @return	string
     */
    public function render()
    {
        $limit     = isset( $this->configuration['donateTopDonors_limit'] ) ? $this->configuration['donateTopDonors_limit'] : 5;
        $timeframe = isset( $this->configuration['donateTopDonors_timeframe'] ) ? $this->configuration['donateTopDonors_timeframe'] : 'month';
        $where     = array( array( 'member_id>0 AND anon=0 AND anon_amount=0' ) );

        /* What's our time frame? */
        if( $timeframe and $timeframe != 'all' )
        {
            switch ( $timeframe )
            {
                case 'week':
                    $where[] = array( 'date>?', \IPS\DateTime::create()->sub( new \DateInterval( 'P1W' ) )->getTimestamp() );
                    break;
                case 'month':
                    $where[] = array( 'date>?', \IPS\DateTime::create()->sub( new \DateInterval( 'P1M' ) )->getTimestamp() );
                    break;
                case 'year':
                    $where[] = array( 'date>?', \IPS\DateTime::create()->sub( new \DateInterval( 'P1Y' ) )->getTimestamp() );
                    break;
            }
        }

        /* Calculate top donors */
        $topDonors = array();
        foreach ( \IPS\Db::i()->select( array( 'member_id', 'amount' ), 'donate_users', $where ) as $donation )
        {
            if ( !isset( $topDonors[ $donation['member_id'] ] ) )
            {
                $topDonors[ $donation['member_id'] ] = $donation['amount'];
            }
            else
            {
                $topDonors[ $donation['member_id'] ] += $donation['amount'];
            }
        }
        arsort( $topDonors );
        $topDonors = \array_slice( $topDonors, 0, $limit, TRUE );

        /* Add member */
        foreach ( \IPS\Db::i()->select( '*', 'core_members', \IPS\Db::i()->in( 'member_id', array_keys( $topDonors ) ) ) as $member )
        {
            \IPS\Member::constructFromData( $member );
        }

        /* Display */
        return $this->output( $topDonors, $timeframe, $limit );
    }
}